class Spinach::Features::TestHowSpinachWorksForFirstTest < Spinach::FeatureSteps
  step 'I have an empty array' do
  end

  step 'I append my first name and my last name to it' do
  end

  step 'I pass it to my super-duper method' do
  end

  step 'the output should contain a formal greeting' do
  end
end
